/*
 Arduino 1 (PA01)
 
  - connects to an MQTT server
  - publishes "hello world" to the topic "pihome"
  - subscribes to the topic "pihome/xxx"
  - controls as many relays as defined
  - turns on/off a specific led when it receives a specific "on"/"off" from the "pihome/xxx" topic
  - multiple arduino's with same generic sketch can run parallel to each other
  - multiple arduino's need each to have a unique ip-addres, unique mac address and unique MQTT client-ID
  - tested on arduino-mega with W5500 ethernet shield
*/

//------------------------------------------------------------------------------

#include <SPI.h>
#include <Ethernet2.h>
#include <PubSubClient.h>

// Set relay variables to Arduino digital pins

//PR01
// pin 0 RX
// pin 1 TX
// pin 4 (SS for SD Card)
int L011 = 5; 
int L012 = 6;
int L013 = 7;
int L014 = 8;
int L021 = 9;
// pin 10 (SS for Ethernet)
int L022 = 11;
int L031 = 12;

//PR02
int L041 = 2;
int L042 = 3;
int L043 = 14;
int L111 = 15;
int L112 = 16;
int L121 = 17;
int L122 = 18;
int L131 = 19;

// pin 20 SDA
// pin 21 SCL

//PR03
int L141 = 23;
int L151 = 25;
int L152 = 27;
int L153 = 29;
int L154 = 31;
int L155 = 33;
int L156 = 35;
int L161 = 37;

//PR04     
int L201 = 22;
int L211 = 24;
int L212 = 26;
int L221 = 28;
int L231 = 30;
int L241 = 32;
//int rez = 34;
//int rez = 36;
	   
//PR05
int H111 = 39;	     
int H121 = 41;
int H131 = 43;
int H151 = 45;
int H152 = 47;
int H201 = 49;
int H211 = 38;
int H221 = 40;

// pin 50 (SS for Ethernet)
// pin 51 (MOSI)
// pin 52 (MISO)
// pin 53 (SCK)

//PR06
//A0 (D54)
int H231 = 54;
int H241 = 55;
// int Rez = 56;
// int Rez = 57;
// int Rez = 58;
// int Rez = 59;

// pin 60 inaccesibble
// int Rez = 61;
// int Rez = 62;
// int Rez = 63;
// int Rez = 64;
// int Rez = 65;
// int Rez = 66;
// int Rez = 67;
// int Rez = 68;
int resetPIN = 69;

//A15 (D69)

// Set variables to act as virtual switches
// Set variable values initially to LOW (and not HIGH)
// HIGH or LOW as defautl is based on Relay Board               

int ValL011 = LOW;
int ValL012 = LOW;
int ValL013 = LOW;
int ValL014 = LOW;
int ValL021 = LOW;
int ValL022 = LOW;
int ValL031 = LOW;
int ValL041 = LOW;
int ValL042 = LOW;
int ValL043 = LOW;
int ValL111 = LOW;
int ValL112 = LOW;
int ValL121 = LOW;
int ValL122 = LOW;
int ValL131 = LOW;
int ValL141 = LOW;
int ValL151 = LOW;
int ValL152 = LOW;
int ValL153 = LOW;
int ValL154 = LOW;
int ValL155 = LOW;
int ValL156 = LOW;
int ValL161 = LOW;
int ValL201 = LOW;
int ValL211 = LOW;
int ValL212 = LOW;
int ValL221 = LOW;
int ValL231 = LOW;
int ValL241 = LOW;

int ValH111 = LOW;
int ValH121 = LOW;
int ValH131 = LOW;
int ValH151 = LOW;
int ValH152 = LOW;
int ValH201 = LOW;
int ValH211 = LOW;
int ValH221 = LOW;
int ValH231  = LOW;
int ValH241  = LOW;

int Valreset = HIGH; 

//---------------------------------------------------------------------------

// Arduino MAC address must be unique for every node in same network
// To make a new unique address change last letter
// Arduino 1
byte mac[]    = { 0xCC, 0xFA, 0x0C, 0xCA, 0x19, 0x01 };  

// Unique static IP address of this Arduino 1
IPAddress ip(10,1,1,211);


// IP Address of your MQTT broker (OpenHAB server)
byte server[] = { 10, 1, 1, 200 };

// Handle and convert incoming MQTT messages ----------------------------------------

void callback(char* topic, byte* payload, unsigned int length) {
  // handle message arrived
  String content="";
  char character;
  for (int num=0;num<length;num++) {
      character = payload[num];
      content.concat(character);
  }   
  
//Relay Power
//Set specific virtual switches on basis of specific incoming messages ---------------------------- 
  if (content == "L011_on")  { ValL011 =  HIGH; }
  if (content == "L011_off") { ValL011 =  LOW; }  
  if (content == "L012_on")  { ValL012 =  HIGH; }
  if (content == "L012_off") { ValL012 =  LOW; }  
  if (content == "L013_on")  { ValL013 =  HIGH; }
  if (content == "L013_off") { ValL013 =  LOW; }  
  if (content == "L014_on")  { ValL014 =  HIGH; }
  if (content == "L014_off") { ValL014 =  LOW; }  
  if (content == "L021_on")  { ValL021 =  HIGH; }
  if (content == "L021_off") { ValL021 =  LOW; }  
  if (content == "L022_on")  { ValL022 =  HIGH; }
  if (content == "L022_off") { ValL022 =  LOW; }  
  if (content == "L031_on")  { ValL031 =  HIGH; }
  if (content == "L031_off") { ValL031 =  LOW; }  
  if (content == "L041_on")  { ValL041 =  HIGH; }
  if (content == "L041_off") { ValL041 =  LOW; }  
  if (content == "L042_on")  { ValL042 =  HIGH; }
  if (content == "L042_off") { ValL042 =  LOW; }  
  if (content == "L043_on")  { ValL043 =  HIGH; }
  if (content == "L043_off") { ValL043 =  LOW; }  
  if (content == "L111_on")  { ValL111 =  HIGH; }
  if (content == "L111_off") { ValL111 =  LOW; }  
  if (content == "L112_on")  { ValL112 =  HIGH; }
  if (content == "L112_off") { ValL112 =  LOW; }  
  if (content == "L121_on")  { ValL121 =  HIGH; }
  if (content == "L121_off") { ValL121 =  LOW; }  
  if (content == "L122_on")  { ValL122 =  HIGH; }
  if (content == "L122_off") { ValL122 =  LOW; }  
  if (content == "L131_on")  { ValL131 =  HIGH; }
  if (content == "L131_off") { ValL131 =  LOW; }  
  if (content == "L141_on")  { ValL141 =  HIGH; }
  if (content == "L141_off") { ValL141 =  LOW; } 
  if (content == "L151_on")  { ValL151 =  HIGH; }
  if (content == "L151_off") { ValL151 =  LOW; }  
  if (content == "L152_on")  { ValL152 =  HIGH; }
  if (content == "L152_off") { ValL152 =  LOW; }  
  if (content == "L153_on")  { ValL153 =  HIGH; }
  if (content == "L153_off") { ValL153 =  LOW; }  
  if (content == "L154_on")  { ValL154 =  HIGH; }
  if (content == "L154_off") { ValL154 =  LOW; }  
  if (content == "L155_on")  { ValL155 =  HIGH; }
  if (content == "L155_off") { ValL155 =  LOW; }  
  if (content == "L156_on")  { ValL156 =  HIGH; }
  if (content == "L156_off") { ValL156 =  LOW; }  
  if (content == "L161_on")  { ValL161 =  HIGH; }
  if (content == "L161_off") { ValL161 =  LOW; }  
  if (content == "L201_on")  { ValL201 =  HIGH; }
  if (content == "L201_off") { ValL201 =  LOW; }   
  if (content == "L211_on")  { ValL211 =  HIGH; }
  if (content == "L211_off") { ValL211 =  LOW; }  
  if (content == "L212_on")  { ValL212 =  HIGH; }
  if (content == "L212_off") { ValL212 =  LOW; }  
  if (content == "L221_on")  { ValL221 =  HIGH; }
  if (content == "L221_off") { ValL221 =  LOW; }  
  if (content == "L231_on")  { ValL231 =  HIGH; }
  if (content == "L231_off") { ValL231 =  LOW; }  
  if (content == "L241_on")  { ValL241 =  HIGH; }
  if (content == "L241_off") { ValL241 =  LOW; } 
  
  if (content == "H111_on")  { ValH111 =  HIGH; }
  if (content == "H111_off") { ValH111 =  LOW; }  
  if (content == "H121_on")  { ValH121 =  HIGH; }
  if (content == "H121_off") { ValH121 =  LOW; }  
  if (content == "H131_on")  { ValH131 =  HIGH; }
  if (content == "H131_off") { ValH131 =  LOW; }  
  if (content == "H151_on")  { ValH151 =  HIGH; }
  if (content == "H151_off") { ValH151 =  LOW; }  
  if (content == "H152_on")  { ValH152 =  HIGH; }
  if (content == "H152_off") { ValH152 =  LOW; }  
  if (content == "H201_on")  { ValH201 =  HIGH; }
  if (content == "H201_off") { ValH201 =  LOW; }  
  if (content == "H211_on")  { ValH211 =  HIGH; }
  if (content == "H211_off") { ValH211 =  LOW; }  
  if (content == "H221_on")  { ValH221 =  HIGH; }
  if (content == "H221_off") { ValH221 =  LOW; }  
  if (content == "H231_on")   { ValH231  =  HIGH; }
  if (content == "H231_off")  { ValH231 =  LOW; }  
  if (content == "H241_on")   { ValH241 =  HIGH; }
  if (content == "H241_off")  { ValH241 =  LOW; } 
  
  //Remote reset feature
 if (content == "resetPA1_on")  { Valreset =  HIGH; }
 if (content == "resetPA1_off") { Valreset =  LOW; }  
 digitalWrite(resetPIN, Valreset); 
  
  
  
 
 

  // Set digital pin states according to virtual switch settings

  digitalWrite(L011, ValL011);
  digitalWrite(L012, ValL012);
  digitalWrite(L013, ValL013);
  digitalWrite(L014, ValL014);
  digitalWrite(L021, ValL021);
  digitalWrite(L022, ValL022);
  digitalWrite(L031, ValL031);
  digitalWrite(L041, ValL041); 
  digitalWrite(L042, ValL042);
  digitalWrite(L043, ValL043);
  digitalWrite(L111, ValL111);
  digitalWrite(L112, ValL112);
  digitalWrite(L121, ValL121);
  digitalWrite(L122, ValL122);
  digitalWrite(L131, ValL131);
  digitalWrite(L141, ValL141); 
  digitalWrite(L151, ValL151);
  digitalWrite(L152, ValL152);
  digitalWrite(L153, ValL153);
  digitalWrite(L154, ValL154);
  digitalWrite(L155, ValL155);
  digitalWrite(L156, ValL156);
  digitalWrite(L161, ValL161);
  digitalWrite(L201, ValL201);
  digitalWrite(L211, ValL211);
  digitalWrite(L212, ValL212);
  digitalWrite(L221, ValL221);
  digitalWrite(L231, ValL231);
  digitalWrite(L241, ValL241);
 
  digitalWrite(H111, ValH111);
  digitalWrite(H121, ValH121);
  digitalWrite(H131, ValH131);
  digitalWrite(H151, ValH151);
  digitalWrite(H152, ValH152);
  digitalWrite(H201, ValH201);
  digitalWrite(H211, ValH211);
  digitalWrite(H221, ValH221);
  digitalWrite(H231, ValH231 );
  digitalWrite(H241, ValH241 );   
}

// Initiate instances -----------------------------------
EthernetClient arduino1;
PubSubClient client(server, 1883, callback, arduino1);

void setup()
{
  // Setup ethernet connection to MQTT broker
  Ethernet.begin(mac, ip);
  if (client.connect("arduino1", "openhabian", "openhabian")) {  							
    client.publish("pihome","Hello world - here Arduino1 with IP 10.1.1.211");
    client.subscribe("pihome/lights");
    client.subscribe("pihome/heating");
    client.subscribe("pihome/diagnostics");
    								
  }
   //Remote reset feature
  digitalWrite(resetPIN, HIGH);  
  pinMode(resetPIN,OUTPUT);	
}

//-----------------------------------------------
long lastReconnectAttempt = 0;

boolean reconnect() {
  if (client.connect("arduino1", "openhabian", "openhabian")) {
    // Once connected, publish an announcement...
    client.publish("pihome","Arduino 1 - reconnected");
    client.subscribe("pihome/lights");
    client.subscribe("pihome/heating");
    client.subscribe("pihome/diagnostics");
  }
  return client.connected();
}
//----------------------------------------------

void loop()
{
  
  if (!client.connected()) {
    long now = millis();
    if (now - lastReconnectAttempt > 5000) {
      lastReconnectAttempt = now;
      // Attempt to reconnect
      if (reconnect()) {
        lastReconnectAttempt = 0;
      }
    }
  } else {
    // Client connected

    client.loop();
}
  
}

// End of sketch ---------------------------------
