/*
 Arduino 2 (PA02)
 
  - connects to an MQTT server
  - publishes "hello world" to the topic "pihome"
  - subscribes to the topic "pihome/xxx"
  - controls as many relays as defined
  - turns on/off a specific led when it receives a specific "on"/"off" from the "pihome/xxx" topic
  - multiple arduino's with same generic sketch can run parallel to each other
  - multiple arduino's need each to have a unique ip-addres, unique mac address and unique MQTT client-ID
  - tested on arduino-mega with W5500 ethernet shield
*/

//------------------------------------------------------------------------------

#include <SPI.h>
#include <Ethernet2.h>
#include <PubSubClient.h>

// Set relay variables to Arduino digital pins

//PR07
// pin 0 RX
// pin 1 TX
// pin 4 (SS for SD Card)
int S011 = 5;
int S012 = 6;
int S013 = 7;	     
int S014 = 8;
int S015 = 9;
// pin 10 (SS for Ethernet)
int S016 = 11;
int S111 = 12;
int R241a = 64; 

//PR08
int S121 = 2;
int S131 = 3;
int S141 = 14;
int S142 = 15;
int S151 = 16;
int S152 = 17;
int S153 = 18;
int S154 = 19;
// pin 20 SDA
// pin 21 SCL

//PR9
int S155  = 23;
int S161  = 25;
int S201  = 27;
int S211  = 29;
int S221  = 31;
int S231  = 33;
int S241  = 35;
int R241b = 37;

//PR10     
int R111a = 22;
int R111b = 24;
int R131a = 26;
int R131b = 28;
int R132a = 30;
int R132b = 32;
int R141a = 34;
int R141b = 36;		   
//PR11     
int R151a = 39;
int R151b = 41;
int R152a = 43;
int R152b = 45;
int R153a = 47;
int R153b = 49;
int R154a = 38;
int R154b = 40; 


// pin 50 (SS for Ethernet)
// pin 51 (MOSI)
// pin 52 (MISO)
// pin 53 (SCK)

//PR12
//A0 (D54)
int R155a = 54;
int R155b = 55;
int R211a = 56;
int R211b = 57;
int R221a = 58;
int R221b = 59; 
// pin 60 inaccesibble 
int R231a = 62; 
int R231b = 63;

//int rez = 65;
//int rez = 66;
//int rez = 67;
//int rez = 68;
int resetPIN = 69;
//A15 (D69)

// Set variables to act as virtual switches
// Set variable values initially to LOW (and not HIGH)
// HIGH or LOW as defautl is based on Relay Board               



int ValS011 = LOW;
int ValS012 = LOW;
int ValS013 = LOW;
int ValS014 = LOW;
int ValS015 = LOW;
int ValS016 = LOW;
int ValS111 = LOW;
int ValS121 = LOW;
int ValS131 = LOW;
int ValS141 = LOW;
int ValS142 = LOW;
int ValS151 = LOW;
int ValS152 = LOW;
int ValS153 = LOW;
int ValS154 = LOW;
int ValS155 = LOW;
int ValS161 = LOW;
int ValS201 = LOW;
int ValS211 = LOW;
int ValS221 = LOW;
int ValS231 = LOW;
int ValS241 = LOW;

int ValR111a = LOW;
int ValR111b = LOW;
int ValR131a = LOW;
int ValR131b = LOW;
int ValR132a = LOW;
int ValR132b = LOW;
int ValR141a = LOW;
int ValR141b = LOW;
int ValR151a = LOW;
int ValR151b = LOW;
int ValR152a = LOW;
int ValR152b = LOW;
int ValR153a = LOW;
int ValR153b = LOW;
int ValR154a = LOW;
int ValR154b = LOW;
int ValR155a = LOW;
int ValR155b = LOW;
int ValR211a = LOW;
int ValR211b = LOW;
int ValR221a = LOW;
int ValR221b = LOW;
int ValR231a = LOW;
int ValR231b = LOW;
int ValR241a = LOW;
int ValR241b = LOW;

int Valreset = HIGH; 

//---------------------------------------------------------------------------

// Arduino MAC address must be unique for every node in same network
// To make a new unique address change last letter
// Arduino 2
byte mac[]    = { 0xCC, 0xFA, 0x05, 0xCB, 0x19, 0x01 };  

// Unique static IP address of this Arduino 1
IPAddress ip(10,1,1,212);


// IP Address of your MQTT broker (OpenHAB server)
byte server[] = { 10, 1, 1, 200 };

// Handle and convert incoming MQTT messages ----------------------------------------



void callback(char* topic, byte* payload, unsigned int length) {
  // handle message arrived
  String content="";
  char character;
  for (int num=0;num<length;num++) {
      character = payload[num];
      content.concat(character);
  }   
  
//Relay Power
// Set specific virtual switches on basis of specific incoming messages ----------------------------
 
  if (content == "S011_on")  { ValS011 =  HIGH; }
  if (content == "S011_off") { ValS011 =  LOW; }  
  if (content == "S012_on")  { ValS012 =  HIGH; }
  if (content == "S012_off") { ValS012 =  LOW; }  
  if (content == "S013_on")  { ValS013 =  HIGH; }
  if (content == "S013_off") { ValS013 =  LOW; }  
  if (content == "S014_on")  { ValS014 =  HIGH; }
  if (content == "S014_off") { ValS014 =  LOW; }  
  if (content == "S015_on")  { ValS015 =  HIGH; }
  if (content == "S015_off") { ValS015 =  LOW; }  
  if (content == "S016_on")  { ValS016 =  HIGH; }
  if (content == "S016_off") { ValS016 =  LOW; }  
  if (content == "S111_on")  { ValS111 =  HIGH; }
  if (content == "S111_off") { ValS111 =  LOW; }  
  if (content == "S121_on")  { ValS121 =  HIGH; }
  if (content == "S121_off") { ValS121 =  LOW; }  
  if (content == "S131_on")  { ValS131 =  HIGH; }
  if (content == "S131_off") { ValS131 =  LOW; }  
  if (content == "S141_on")  { ValS141 =  HIGH; }
  if (content == "S141_off") { ValS141 =  LOW; }  
  if (content == "S142_on")  { ValS142 =  HIGH; }
  if (content == "S142_off") { ValS142 =  LOW; }  
  if (content == "S151_on")  { ValS151 =  HIGH; }
  if (content == "S151_off") { ValS151 =  LOW; }  
  if (content == "S152_on")  { ValS152 =  HIGH; }
  if (content == "S152_off") { ValS152 =  LOW; }  
  if (content == "S153_on")  { ValS153 =  HIGH; }
  if (content == "S153_off") { ValS153 =  LOW; }  
  if (content == "S154_on")  { ValS154 =  HIGH; }
  if (content == "S154_off") { ValS154 =  LOW; }  
  if (content == "S155_on")  { ValS155 =  HIGH; }
  if (content == "S155_off") { ValS155 =  LOW; }  
  if (content == "S161_on")  { ValS161 =  HIGH; }
  if (content == "S161_off") { ValS161 =  LOW; }  
  if (content == "S201_on")  { ValS201 =  HIGH; }
  if (content == "S201_off") { ValS201 =  LOW; }  
  if (content == "S211_on")  { ValS211 =  HIGH; }
  if (content == "S211_off") { ValS211 =  LOW; }  
  if (content == "S221_on")  { ValS221 =  HIGH; }
  if (content == "S221_off") { ValS221 =  LOW; }  
  if (content == "S231_on")  { ValS231 =  HIGH; }
  if (content == "S231_off") { ValS231 =  LOW; }  
  if (content == "S241_on")  { ValS241 =  HIGH; }
  if (content == "S241_off") { ValS241 =  LOW; } 
  
  
  if (content == "R111a_on")  { ValR111b =  LOW; ValR111a =  HIGH; }
  if (content == "R111b_on")  { ValR111a =  LOW; ValR111b =  HIGH; }
  if (content == "R111_off")  { ValR111a =  LOW; ValR111b =  LOW; }  
  if (content == "R131a_on")  { ValR131b =  LOW; ValR131a =  HIGH; } 
  if (content == "R131b_on")  { ValR131a =  LOW; ValR131b =  HIGH; }
  if (content == "R131_off")  { ValR131a =  LOW; ValR131b =  LOW; }  
  if (content == "R132a_on")  { ValR132b =  LOW; ValR132a =  HIGH; }
  if (content == "R132b_on")  { ValR132a =  LOW; ValR132b =  HIGH; }
  if (content == "R132_off")  { ValR132a =  LOW; ValR132b =  LOW; } 
  if (content == "R141a_on")  { ValR141b =  LOW; ValR141a =  HIGH; } 
  if (content == "R141b_on")  { ValR141a =  LOW; ValR141b =  HIGH; }
  if (content == "R141_off")  { ValR141a =  LOW; ValR141b =  LOW; }  
  if (content == "R151a_on")  { ValR151b =  LOW; ValR151a =  HIGH; }
  if (content == "R151b_on")  { ValR151a =  LOW; ValR151b =  HIGH; }
  if (content == "R151_off")  { ValR151a =  LOW; ValR151b =  LOW; }  
  if (content == "R152a_on")  { ValR152b =  LOW; ValR152a =  HIGH; } 
  if (content == "R152b_on")  { ValR152a =  LOW; ValR152b =  HIGH; }
  if (content == "R152_off")  { ValR152a =  LOW; ValR152b =  LOW; }  
  if (content == "R153a_on")  { ValR153b =  LOW; ValR153a =  HIGH; }
  if (content == "R153b_on")  { ValR153a =  LOW; ValR153b =  HIGH; }
  if (content == "R153_off")  { ValR153a =  LOW; ValR153b =  LOW; }    
  if (content == "R154a_on")  { ValR154b =  LOW; ValR154a =  HIGH; } 
  if (content == "R154b_on")  { ValR154a =  LOW; ValR154b =  HIGH; }
  if (content == "R154_off")  { ValR154a =  LOW; ValR154b =  LOW; }  
  if (content == "R155a_on")  { ValR155b =  LOW; ValR155a =  HIGH; } 
  if (content == "R155b_on")  { ValR155a =  LOW; ValR155b =  HIGH; }
  if (content == "R155_off")  { ValR155a =  LOW; ValR155b =  LOW; }  
  if (content == "R211a_on")  { ValR211b =  LOW; ValR211a =  HIGH; }
  if (content == "R211b_on")  { ValR211a =  LOW; ValR211b =  HIGH; }
  if (content == "R211_off")  { ValR211a =  LOW; ValR211b =  LOW; }  
  if (content == "R221a_on")  { ValR221b =  LOW; ValR221a =  HIGH; }  
  if (content == "R221b_on")  { ValR221a =  LOW; ValR221b =  HIGH; }
  if (content == "R221_off")  { ValR221a =  LOW; ValR221b =  LOW; }
  if (content == "R231a_on")  { ValR231b =  LOW; ValR231a =  HIGH; }  
  if (content == "R231b_on")  { ValR231a =  LOW; ValR231b =  HIGH; }
  if (content == "R231_off")  { ValR231a =  LOW; ValR231b =  LOW; }  
  if (content == "R241a_on")  { ValR241b =  LOW; ValR241a =  HIGH; }  
  if (content == "R241b_on")  { ValR241a =  LOW; ValR241b =  HIGH; }
  if (content == "R241_off")  { ValR241a =  LOW; ValR241b =  LOW; }    

   //Remote reset feature
 if (content == "resetPA2_on")  { Valreset =  HIGH; }
 if (content == "resetPA2_off") { Valreset =  LOW; }  
 digitalWrite(resetPIN, Valreset); 
  
 

  // Set digital pin states according to virtual switch settings
  digitalWrite(S011, ValS011);
  digitalWrite(S012, ValS012);
  digitalWrite(S013, ValS013);
  digitalWrite(S014, ValS014);
  digitalWrite(S015, ValS015);
  digitalWrite(S016, ValS016);
  digitalWrite(S111, ValS111);
  digitalWrite(S121, ValS121);
  digitalWrite(S131, ValS131);
  digitalWrite(S141, ValS141);
  digitalWrite(S142, ValS142);
  digitalWrite(S151, ValS151);
  digitalWrite(S152, ValS152);
  digitalWrite(S153, ValS153);
  digitalWrite(S154, ValS154);
  digitalWrite(S155, ValS155);
  digitalWrite(S161, ValS161);
  digitalWrite(S201, ValS201);
  digitalWrite(S211, ValS211);
  digitalWrite(S221, ValS221);
  digitalWrite(S231, ValS231);
  digitalWrite(S241, ValS241);
  
  digitalWrite(R111a, ValR111a);
  digitalWrite(R111b, ValR111b);
  digitalWrite(R131a, ValR131a);
  digitalWrite(R131b, ValR131b);
  digitalWrite(R132a, ValR132a);
  digitalWrite(R132b, ValR132b);  
  digitalWrite(R141a, ValR141a);
  digitalWrite(R141b, ValR141b);
  digitalWrite(R151a, ValR151a);
  digitalWrite(R151b, ValR151b);
  digitalWrite(R152a, ValR152a);
  digitalWrite(R152b, ValR152b);
  digitalWrite(R153a, ValR153a);
  digitalWrite(R153b, ValR153b);
  digitalWrite(R154a, ValR154a);
  digitalWrite(R154b, ValR154b);
  digitalWrite(R155a, ValR155a);
  digitalWrite(R155b, ValR155b);
  digitalWrite(R211a, ValR211a);
  digitalWrite(R211b, ValR211b);
  digitalWrite(R221a, ValR221a);
  digitalWrite(R221b, ValR221b);
  digitalWrite(R231a, ValR231a);
  digitalWrite(R231b, ValR231b);
  digitalWrite(R241a, ValR241a);
  digitalWrite(R241b, ValR241b);  
}

// Initiate instances -----------------------------------

EthernetClient arduino2;
PubSubClient client(server, 1883, callback, arduino2);




void setup()
{
  // Setup ethernet connection to MQTT broker
  Ethernet.begin(mac, ip);
  if (client.connect("arduino2", "openhabian", "openhabian")) {  							
    client.publish("pihome","Hello world - here Arduino2 with IP 10.1.1.212");
    client.subscribe("pihome/sockets");
    client.subscribe("pihome/shutters");
    client.subscribe("pihome/diagnostics");
    								
  }
  //Remote reset feature
  digitalWrite(resetPIN, HIGH);  
  pinMode(resetPIN,OUTPUT);	
}


//-----------------------------------------------
long lastReconnectAttempt = 0;

boolean reconnect() {
  if (client.connect("arduino2", "openhabian", "openhabian")) {
    // Once connected, publish an announcement...
    client.publish("pihome","Arduino 2 - reconnected");
    client.subscribe("pihome/sockets");
    client.subscribe("pihome/shutters");
    client.subscribe("pihome/diagnostics");
  }
  return client.connected();
}
//----------------------------------------------


void loop()
{
  
  if (!client.connected()) {
    long now = millis();
    if (now - lastReconnectAttempt > 5000) {
      lastReconnectAttempt = now;
      // Attempt to reconnect
      if (reconnect()) {
        lastReconnectAttempt = 0;
      }
    }
  } else {
    // Client connected

    client.loop();
}
  
}

// End of sketch ---------------------------------
