/*
 Arduino 3 (PA03)
 
  - connects to an MQTT server
  - publishes "hello world" to the topic "pihome"
  - subscribes to the specific topic pihome/xxx
  - sends a specific "on"/"off" to the specific topic when a specific button is pressed
  - multiple arduino's with same generic sketch can run parallel to each other
  - multiple arduino's need each to have a unique ip-addres, unique mac address and unique MQTT client-ID
  - tested on arduino-mega with W5500 ethernet shield
*/

//------------------------------------------------------------------------------

#include <SPI.h>
#include <Ethernet2.h>
#include <PubSubClient.h>

// pin 0 RX
// pin 1 TX
// pin 4 (SS for SD Card)
//B111 has underscore because B111 variable already exist in some of the library :D and PIN 5 is not usable as well
int B111_ = 6; 
int B112  = 7;
int B121  = 8;
int B122  = 9;
// pin 10 (SS for Ethernet)
int B131  = 11;
int B132  = 12;
int B133  = 13;
int B134  = 2;
int B141 = 3;
int B142 = 14;
int B151 = 15;
int B152 = 16;
int B153 = 17;
int B154 = 18;
int B155 = 19;

// pin 20 SDA
// pin 21 SCL

int B156 = 23;
int B157 = 25;
int B158 = 27;
int B159 = 29;
int B160 = 31;
int B161 = 33;
int B162 = 35;
int B201 = 37;
int B202 = 39;
int B211 = 41;
int B212 = 43;
int B213 = 45;
int B214 = 47;
int B221 = 49;

int B222 = 22;
int B223 = 24;
int B224 = 26;		   
int B231 = 28;
int B232 = 30;
int B233 = 32;
int B234 = 34;
int B241 = 36;
int B242 = 38;
int B243 = 40;
int B244 = 42;
     
//int rez = 44;
//int rez = 46;
//int rez = 48;

// pin 50 (SS for Ethernet)
// pin 51 (MOSI)
// pin 52 (MISO)
// pin 53 (SCK)

//A0 (D54)
// int Rez = 54;
// int Rez = 55;
// int Rez = 56;
// int Rez = 57;
// int Rez = 58;
// int Rez = 59;
// pin 60 inaccesibble
// int Rez = 61;

//int rez = 63;
//int rez = 64;
//int rez = 65;
//int rez = 66;
//int rez = 67;
//int rez = 68;
//int rez = 69;
//A15 (D69)

// Arduino MAC address must be unique for every node in the same network
// To make a new unique address change the last letter
// Arduino 3
byte mac[]    = { 0xCB, 0xFA, 0x0B, 0xCC, 0x19, 0x01 };  

// Unique static IP address of this Arduino 1
IPAddress ip(10, 1, 1, 213);

// IP Address of your MQTT broker (OpenHAB server)
byte server[] = { 10, 1, 1, 200 };

// Handle and convert incoming MQTT messages
void callback(char* topic, byte* payload, unsigned int length) {
  // handle message arrived (no messages expected though)
}

// Initiate instances
EthernetClient arduino3;
PubSubClient client(server, 1883, callback, arduino3);

void setup() {
  Ethernet.begin(mac, ip);
  if (client.connect("arduino3", "openhabian", "openhabian")) {
    client.publish("pihome", "Hello world - here Arduino PA03 with IP 10.1.1.213");
  }
  
  // Set pin modes
  int pins[] = {B111_, B112, B121, B122, B131, B132, B133, B134, B141, B142, B151, B152, B153, B154, B155, 
                B156, B157, B158, B159, B160, B161, B162, B201, B202, B211, B212, B213, B214, B221, B222, 
                B223, B224, B231, B232, B233, B234, B241, B242, B243, B244};
                
  for (int i = 0; i < sizeof(pins) / sizeof(pins[0]); i++) {
    pinMode(pins[i], INPUT);
    digitalWrite(pins[i], HIGH);  // Enable pull-up resistor
  }
}

// Debounce settings
unsigned long debounceDelay = 50;  // the debounce time in milliseconds
unsigned long lastDebounceTime[40]; // to store the last debounce time for each button
bool lastButtonState[40]; // to store the last button state
bool buttonState[40]; // to store the current button state

long lastReconnectAttempt = 0;

boolean reconnect() {
  if (client.connect("arduino3", "openhabian", "openhabian")) {
    client.publish("pihome", "Arduino 3 - reconnected");
  }
  return client.connected();
}

void loop() {
  // Button check and debounce
  int pins[] = {B111_, B112, B121, B122, B131, B132, B133, B134, B141, B142, B151, B152, B153, B154, B155, 
                B156, B157, B158, B159, B160, B161, B162, B201, B202, B211, B212, B213, B214, B221, B222, 
                B223, B224, B231, B232, B233, B234, B241, B242, B243, B244};
  char* buttonNames[] = {"B111_push", "B112_push", "B121_push", "B122_push", "B131_push", "B132_push", "B133_push", "B134_push", "B141_push", "B142_push", "B151_push", "B152_push", "B153_push", "B154_push", "B155_push", 
                         "B156_push", "B157_push", "B158_push", "B159_push", "B160_push", "B161_push", "B162_push", "B201_push", "B202_push", "B211_push", "B212_push", "B213_push", "B214_push", "B221_push", "B222_push", 
                         "B223_push", "B224_push", "B231_push", "B232_push", "B233_push", "B234_push", "B241_push", "B242_push", "B243_push", "B244_push"};

  for (int i = 0; i < sizeof(pins) / sizeof(pins[0]); i++) {
    bool reading = digitalRead(pins[i]) == LOW;
    
    if (reading != lastButtonState[i]) {
      lastDebounceTime[i] = millis();
    }
    
    if ((millis() - lastDebounceTime[i]) > debounceDelay) {
      if (reading != buttonState[i]) {
        buttonState[i] = reading;
        if (buttonState[i]) {
          client.publish("pihome/buttons", buttonNames[i]);
        }
      }
    }
    
    lastButtonState[i] = reading;
  }

  if (!client.connected()) {
    long now = millis();
    if (now - lastReconnectAttempt > 5000) {
      lastReconnectAttempt = now;
      if (reconnect()) {
        lastReconnectAttempt = 0;
      }
    }
  } else {
    client.loop();
  }
}
